#!/bin/bash
set -e

{% set instance_data = jsonMap(container.instanceData()) %}

# Files created at run-time should be group-writable
umask 0002

if [[ $(id -u) -eq 0 ]]; then
    echo "ERROR: enterprise_search.sh running as uid 0 (root), exiting."
    exit 1
fi

# A WORKAROUND TO GET GC LOGS IN PLACE
# Ref: https://github.com/elastic/cloud/issues/68802#issuecomment-742567764
export JAVA_GC_LOG_DIR=/app/logs

run_beat () {
  BEAT_NAME=${1}
  while true; do
    /opt/${BEAT_NAME}/${BEAT_NAME} --path.config /app/config/${BEAT_NAME} --path.data /tmp/${BEAT_NAME}-data
    echo "${BEAT_NAME} exited with code $?. Re-running in 2 seconds"
    sleep 2
  done
}

{% if instance_data.observability.logging is defined %}
echo "Starting filebeat"
run_beat filebeat &
{% endif %}
{% if instance_data.observability.metrics is defined %}
echo "Starting metricbeat"
run_beat metricbeat &
{% endif %}

# This allows running with Elasticsearch on 8.0.0-SNAPSHOT version
# Ref: https://github.com/elastic/ent-search/pull/3102
export ALLOW_PREVIEW_ELASTICSEARCH_8X=true
export JAVA_OPTS="-Xms{{ container.heapSize() }}M -Xmx{{ container.heapSize() }}M -XX:OnOutOfMemoryError=/app/on-error-exitcode.sh"
export FILEBEAT_JAVA_OPTS="-Xmx256M -Xms8M"
export ENT_SEARCH_CONFIG_PATH=/app/config/enterprise_search.yml

{% if heapDumpOnOOM|default(false) %}
if [ -d /app/heap_dumps ]; then
  echo "Enabling heap dump on oom"
  export JAVA_OPTS="$JAVA_OPTS -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/app/heap_dumps"
fi
{% endif %}

# Do not abort script if Enterprise Search returns error code
set +e

_term() {
   echo "Caught SIGTERM"
   kill -TERM $(cat /app/es.pid)
}

trap _term TERM

/usr/share/enterprise-search/bin/enterprise-search all &

ENT_SEARCH_PID=$!
wait ${ENT_SEARCH_PID}
ENT_SEARCH_EXIT_CODE=$?
trap - TERM
wait ${ENT_SEARCH_PID}
# The double wait above is deliberate as the first one returns immediately after TERM signal gets trapped.
# Ref: https://github.com/elastic/cloud-assets/pull/451
if [ "$ENT_SEARCH_EXIT_CODE" -ge "1" ]; then
    echo "Enterprise Search exited with status $ENT_SEARCH_EXIT_CODE, running /app/on-error-exitcode.sh $ENT_SEARCH_EXIT_CODE"
    /app/on-error-exitcode.sh $ENT_SEARCH_EXIT_CODE
fi
exit $ENT_SEARCH_EXIT_CODE
